#!/bin/sh
# $Id: getips.sh,v 1.7 2013/09/17 15:40:31 ca Exp $
#
# Copyright (c) 2011, 2013 Claus Assmann
#
# By using this file, you agree to the terms and conditions set
# forth in the license/LICENSE.3C file which can be found at the
# top level of this source code distribution.

# ---------------------------
# try to retrieve all local IP addresses
# ---------------------------

IFO=if.$$
SD=`dirname $0`
[ X"$SD" = "X" ] && SD=.
. $SD/common.sh

GETIPV4=1
GETIPV6=0

while getopts 46 FLAG
do
  case "${FLAG}" in
    4) GETIPV4=0;;
    6) GETIPV6=1;;
    ?)
       echo "$0: unknown option ${FLAG}"
       echo "$0: options:"
       echo "-4	do NOT list IPv4 addresses"
       echo "-6	list IPv6 addresses"
       echo "by default only IPv4 addresses are listed"
       exit 1
       ;;
  esac
done
shift `expr ${OPTIND} - 1`

if ifconfig -a > $IFO
then
  if grep '^[a-z][a-z]*[0-9:]*: flags=' $IFO >/dev/null
  then
    # OpenBSD, SunOS 5, ...
    awk -v getipv6=${GETIPV6} -v getipv4=${GETIPV4} '
BEGIN { nic=""; }
/^[a-z][a-z]*[0-9:]*: flags=/ {
  if ($0 ~ "UP") {
    nic=$1;
# print "found ", nic;
  }
  else {
# print "down ", $1;
    nic="";
  }
}
/inet / { if (nic != "") {
    if (getipv4) {
      #print "nic=", nic, "ipv4=", $2;
      print $2;
    }
  }
}
/inet6 / { if (nic != "") {
    if (getipv6) {
      ip6 = $2;
      p = match(ip6, "[%/]");
      if (p > 2) {
        ip6=substr(ip6, 1, p - 1);
      }
      #print "nic=", nic, "ipv6=", ip6;
      print ip6;
    }
  }
}
' < $IFO
   r=0
  elif grep '^[a-z][a-z0-9]*[0-9:]*  *Link ' $IFO >/dev/null
  then
    # Linux
    awk -v getipv6=${GETIPV6} -v getipv4=${GETIPV4} '
BEGIN { nic=""; up=0; addr=""; }
/^[a-z][a-z0-9]*[0-9:]*  *Link/ {
  if (nic != "" && up && addr != "") {
     print addr;
  }
  nic=$1;
  up=0;
  addr="";
#print "nic",nic;
}
/^ *UP / {
    if (nic != "") {
      up=1;
#print "nic",nic, "up", "addr", addr;
    }
}
/inet / {
  if (nic != "") {
    if (getipv4) {
      addr=$2;
      gsub("addr:", "", addr);
#print "nic",nic, "addr", addr;
    }
  }
}
/inet6 addr: / {
   if (nic != "") {
    if (getipv6) {
      ip6 = $3;
      p = match(ip6, "/");
      if (p > 2) {
        ip6=substr(ip6, 1, p - 1);
      }
      #print "nic=", nic, "ipv6=", ip6;
      print ip6;
    }
  }
  next;
}
/inet6 / { if (nic != "") {
    if (getipv6) {
      addr6=$2;
      gsub("addr: *", "", addr6);
      # print addr6;
    }
  }
}
END { if (nic != "" && up && addr != "") { print addr; }
}
' < $IFO
   r=0
  else
    echo "$0: ifconfig -a: unknown format"
    r=1
  fi

else
  echo "$0: ifconfig -a failed"
  r=1
fi
rm -f $IFO
