#!/bin/sh
# $Id: smtp.sh,v 1.65 2013/08/24 17:02:38 ca Exp $
# Copyright (c) 2003-2006 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
#
# test program running the full system:
# qmgr, smar, smtpc, smtps
# using smtps and smtpc from statethreads/examples as source/sink
#
test -s stop && exit 1
#
# source directory of this (and other) shell script
SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
#
# default settings
TAS=1
THREADS=1
RCPTS=1
TOTAL=1
MSGS=""
INIT=true
RMDEFEDB=true
RMIBDB=true
WAIT=""
# debug flags
D="-d 5"
# server options
SRVOPTS=${MTA_SERVER_OPTIONS}
CLTOPTS=""
QOPTS=""
ERRCHK=true

# works only with ksh on OSF/1
#if test X"`uname`" != "XOSF1"
#then
while getopts c:d:DkMNn:r:s:T:t:V:w:Q: FLAG
do
  case "${FLAG}" in
    c) CLTOPTS="${OPTARG}";;
    d) D="-d ${OPTARG}";;
    D) INIT=false;;
    M) RMDEFEDB=false;;
    k) RMIBDB=false;;
    N) ERRCHK=false;;
    n) RCPTS=${OPTARG}
		;;
    r) TR=${OPTARG};;
    s) MSGS=${OPTARG};;
    T) TAS=${OPTARG};;
    t) THREADS=${OPTARG};;
    V) SRVOPTS=${OPTARG};;
    w) WAIT=${OPTARG};;
    Q) QOPTS="${OPTARG}";;
    ?)
       echo "$0: unknown option ${FLAG}"
       echo "$0: options:"
       echo "-d n	set debug level"
       echo "-N 	do not check logs for errors"
       echo "-n n	n recipients per transaction"
       echo "-r n	n recipients expected in sink"
       echo "-s n	send n messages with 1..n as body"
       echo "-T n	n transactions"
       echo "-t n	n threads"
       echo "-V opts	additional SMTP server options"
       echo "-w n	wait n seconds after sending messages for delivery"
       echo "-Q opts	additional qmgr options"
       exit 1
       ;;
  esac
done
shift `expr ${OPTIND} - 1`
if test "X"${MSGS} != "X"
then
  TOTAL="${MSGS}"
else
  TOTAL=`expr ${TAS} \* ${THREADS}`
fi
[ x"$TR" = "x" ] && TR=`expr ${RCPTS} \* ${TOTAL}`

# how long to wait for the MTS to deliver all mails?
# maybe make it dependent on number of rcpts/transactions/threads?
# it seems the system is becoming slower :-(
if test "X"${WAIT} = "X"
then
  WAIT=`expr ${RCPTS} + 1 `
fi

# setup/initialize system
. ${SD}/mta-setup.sh
#fi
#

killit()
{
if test -s ${PIDS}
then
  # stop MTA components
  for i in `cat ${PIDS}`
  do
    kill ${i}
  done
fi
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall qmgr smar
fi
if test X"${SRVPID}" != "X"
then
  kill -TERM ${SRVPID}
fi
exit 1
}

SRVPID=""
# store the PIDs for the started programs
rm -f ${PIDS}
trap killit 2

# start SMTP sink
${STE}/smtps2 ${PIPELINING} -b ${H}:${SNKPORT} > ${SRVL} 2>&1 &
status=$?
SRVPID=$!
if test "x${status}" != "x0"
then
  echo "$0: FAIL: cannot start server ${status}"
  exit ${status}
fi
sleep 1

# version of output files
V=0
# path to programs
P=..
rm -f ${SMTPCSOCK}
# file to override settings
# test -f options && . ./options
# go for it
${P}/smar/smar ${D} $@ > a${V}.log 2>&1 &
echo $! >> ${PIDS}
# ${CHKD}/qmgr-running.sh "${SMARSOCK}" || killit
${P}/qmgr/qmgr -c 40 -w ${W4S} -W ${W4C} ${D} ${QOPTS} $@ > q${V}.log 2>&1 &
echo $! >> ${PIDS}
# wait a bit for startup such that the sockets are created
# export SMTPCSOCK
# ${CHKD}/qmgr-running.sh || killit

${P}/smtpc/smtpc -w ${W4S} ${CLTOPTS} -l . -P ${SNKPORT} ${D} $@ > c${V}.log 2>&1 &
echo $! >> ${PIDS}
${P}/smtps/smtps -w ${W4S} ${SRVOPTS} -l . ${D} -b localhost:${SRVPORT} $@ > s${V}.log 2>&1 &
echo $! >> ${PIDS}

# check that the MTA is running
export SRVPORT P
${CHKD}/mta-running.sh || killit

# send messages
if ${STE}/smtpc -s ${TOTAL} -r ${H}:${SRVPORT} -n "${RCPTS}" -T "${TAS}" -t "${THREADS}" > ${CLTL} 2>&1
then
  :
else
  cat ${CLTL}
  ERRS=`expr ${ERRS} + 1 `
fi

sleep ${WAIT}

# stop MTA components
for i in `cat ${PIDS}`
do
  kill ${i}
done
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall qmgr smar
fi

# send SMTP server a signal to dump its state
if kill -USR1 ${SRVPID}
then
  :
else
  echo "kill -USR1 failed: $?"
  exit 1
fi
sleep 2

# check state of SMTP server
if ${GREP} "^Address *${H}:${SNKPORT}" ${SRVL} > /dev/null
then
  :
else
  echo "failed to listen on ${H}:${SNKPORT}"
  ERRS=`expr ${ERRS} + 1 `
fi
if ${GREP} "^Thread.* 8/" ${SRVL} > /dev/null
then
  :
else
  echo "failed to start threads"
  ERRS=`expr ${ERRS} + 1 `
fi
# the number of sessions can vary based on the scheduler...
RQS=`${GREP} "^Sessions served *" ${SRVL} | sed -e 's/^Sessions served *//' | head -n1`
if test "${RQS}" -lt "${TOTAL}"
then
  echo "wrong number of sessions: expected=${TOTAL} got=${RQS}"
  ERRS=`expr ${ERRS} + 1 `
fi
if ${GREP} "^Recipients *${TR}$" ${SRVL} > /dev/null
then
  :
else
  echo "wrong number of recipients: expected=${TR}"
  ${GREP} "^Recipients" ${SRVL}
  ERRS=`expr ${ERRS} + 1 `
fi
if ${GREP} "total=${TOTAL} " ${CLTL} > /dev/null
then
  :
else
  echo "wrong number of mails sent: expected=${TOTAL}"
  ${GREP} "total=" ${CLTL}
  ERRS=`expr ${ERRS} + 1 `
fi

# ----------------
kill -TERM ${SRVPID}

# ----------------
if ${IDBR0} >/dev/null
then
  :
else
  echo "t-idbr-0 failed"
  ERRS=`expr ${ERRS} + 1 `
fi
# ----------------
if ${ERRCHK}
then
for i in a c q s
do
  if egrep "${ERRORPAT}" ${i}${V}.log
  then
    ERRS=`expr ${ERRS} + 1 `
  fi
done
fi

# ----------------
# remove IBDB only if no errors
${RMIBDB} && test ${ERRS} -eq 0 && rm -f ibdd/ibd00000000*

# ----------------
if test ${ERRS} -ne 0
then
  echo "${ERRS} error(s)"
  exit 1
fi
exit 0
