/*
 * Copyright (c) 2010 Sendmail, Inc. and its suppliers.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: readkey.c,v 1.7 2011/01/30 21:49:13 ca Exp $")
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/fcntl.h"
#include "sm/stat.h"
#include "sm/misc.h"
#include "sm/dkim.h"
#include "dkim.h"

#if MTA_USE_DKIM

/*
**  DKIM_READKEY -- read a DKIM key
**
**	Parameters:
**		dkim_s -- DKIM selector context
**
**	Returns:
**		usual error code
*/

sm_ret_T
dkim_readkey(dkim_s_P dkim_s)
{
	sm_ret_T ret;

	SM_REQUIRE(dkim_s != NULL);

	dkim_s->dks_bio_key = NULL;
	dkim_s->dks_rsa = NULL;
	ret = readfile(dkim_s->dks_keyfile, &(dkim_s->dks_priv_key),
			&(dkim_s->dks_priv_key_len));
	if (sm_is_err(ret))
		return ret;	/* modify error code? */

	dkim_s->dks_bio_key = BIO_new_mem_buf(dkim_s->dks_priv_key,
		dkim_s->dks_priv_key_len);
	if (NULL == dkim_s->dks_bio_key) {
		ret = sm_error_perm(SM_EM_DKIM, ENOMEM); /* ? */
		goto error;
	}
	dkim_s->dks_rsa_pkey = PEM_read_bio_PrivateKey(dkim_s->dks_bio_key,
			NULL, NULL, NULL);
	if (NULL == dkim_s->dks_rsa_pkey) {
		ret = sm_error_perm(SM_EM_DKIM, EINVAL); /* ? */
		goto error;
	}
	dkim_s->dks_rsa = EVP_PKEY_get1_RSA(dkim_s->dks_rsa_pkey);
	if (NULL == dkim_s->dks_rsa) {
		ret = sm_error_perm(SM_EM_DKIM, EINVAL); /* ? */
		goto error;
	}
	dkim_s->dks_rsa_pad = RSA_PKCS1_PADDING;
	dkim_s->dks_rsa_keysize = RSA_size(dkim_s->dks_rsa);

	return SM_SUCCESS;

  error:
	if (dkim_s->dks_rsa != NULL) {
		RSA_free(dkim_s->dks_rsa);
		dkim_s->dks_rsa = NULL;
	}
	if (dkim_s->dks_bio_key != NULL) {
		BIO_free(dkim_s->dks_bio_key);
		dkim_s->dks_bio_key = NULL;
	}
	return ret;
}

#endif /* MTA_USE_DKIM */
