/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: sm_inet_ipv62arpa.c,v 1.4 2014/01/30 16:37:29 ca Exp $")

#include "sm/assert.h"
#include "sm/error.h"
#include "sm/ctype.h"
#include "sm/net.h"
#include "sm/str.h"
#include "sm/io.h"

#if MTA_NETINET6
/*
**  SM_INET_IPV62ARPASTR -- convert an IPv6 address to IP6.ARPA format
**
**	Parameters:
**		ipv6 -- IPv6 address
**		dst -- append IP6 ARPA ASCII representation (output)
**
**	Returns:
**		usual sm_error
*/

static char hex_digits[] =
{ '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };

sm_ret_T
sm_inet_ipv62arpastr(ipv6_P ipv6, sm_str_P dst)
{
	uchar *src;
	int i;

	src = (uchar *) ipv6;
	for (i = 15; i >= 0; i--) {
		SM_STR_PUT(dst, hex_digits[src[i] & 0x0f]);
		SM_STR_PUT(dst, '.');
		SM_STR_PUT(dst, hex_digits[(src[i] >> 4) & 0x0f]);
		SM_STR_PUT(dst, '.');
	}
	sm_str_scat(dst, "ip6.arpa");
	return SM_SUCCESS;
}

/*
**  SM_INET_IN6ADDR2ARPASTR -- convert an IPv6 address struct to IP6.ARPA format
**
**	Parameters:
**		inaddr -- IPv6 address struct
**		dst -- append IP6 ARPA ASCII representation (output)
**
**	Returns:
**		usual sm_error
*/

sm_ret_T
sm_inet_in6addr2arpastr(struct in6_addr in6addr, sm_str_P dst)
{
	uchar *src;
	int i;

	src = (uchar *) &in6addr;
	for (i = 15; i >= 0; i--) {
		SM_STR_PUT(dst, hex_digits[src[i] & 0x0f]);
		SM_STR_PUT(dst, '.');
		SM_STR_PUT(dst, hex_digits[(src[i] >> 4) & 0x0f]);
		SM_STR_PUT(dst, '.');
	}
	sm_str_scat(dst, "ip6.arpa");
	return SM_SUCCESS;
}
#endif /* MTA_NETINET6 */
