
#define MAPC_TYPE1	"bhtable"
#define MAPC_NAME1	"bht1"
#define KEY1_1	"left1_1"
#define KEY1_2	"left1_2"
#define KEY1_3	"left1_3"
#define DATA1_1	"right1_1"
#define DATA1_2	"right1_2"

#define MAPC_TYPE2	"hash"
#define MAPC_NAME2	"bdb2"
#define MAPC_FILE2	"./bdb2.db"
#define KEY2_1	"left2_1"
#define KEY2_2	"left2_2"
#define KEY2_3	"left2_3"
#define DATA2_1	"right2_1"
#define DATA2_2	"right2_2"

#define MAPC_TYPE3	"sequence"
#define MAPC_NAME3	"seq1"

#if !T_SEQ_COMMON_NOBDB
static sm_ret_T
crt_bdb_map(sm_maps_P maps, sm_map_P *pmap, const char *name, const char *type, const char *filename)
{
	sm_ret_T ret;
	sm_map_P map;
	sm_cstr_P mtype, mname;

	SM_REQUIRE(maps != NULL);
	SM_REQUIRE(pmap != NULL);
	SM_REQUIRE(name != NULL);
	SM_REQUIRE(type != NULL);

	mtype = mname = NULL;
	mtype = sm_cstr_scpyn0((const uchar *)type, strlen(type));
	ret = sm_error_temp(SM_EM_MAP, ENOMEM);
	SM_TEST_E(mtype != NULL);
	mname = sm_cstr_scpyn0((const uchar *)name, strlen(name));
	SM_TEST_E(mname != NULL);
	ret = sm_bdb_class_create(maps);
	SM_TEST(sm_is_success(ret));
	(void) unlink(filename);

	map = NULL;
	ret = sm_map_open(maps, mname, mtype, SMAP_MODE_CREATE, filename,
			SMAP_MODE_RDWR, &map, SMPO_END);
	SM_TEST_E(sm_is_success(ret));

	*pmap = map;
	return ret;

  error:
	return ret;
}
#endif /* !T_SEQ_COMMON_NOBDB */

static sm_ret_T
crt_map(sm_maps_P maps, sm_map_P *pmap, const char *name, const char *type)
{
	sm_ret_T ret;
	sm_cstr_P mtype, mname;
	sm_map_P map;

	SM_REQUIRE(maps != NULL);
	SM_REQUIRE(pmap != NULL);
	SM_REQUIRE(name != NULL);
	SM_REQUIRE(type != NULL);

	mtype = mname = NULL;
	mtype = sm_cstr_scpyn0((const uchar *)type, strlen(type));
	ret = sm_error_temp(SM_EM_MAP, ENOMEM);
	SM_TEST_E(mtype != NULL);
	mname = sm_cstr_scpyn0((const uchar *)name, strlen(name));
	SM_TEST_E(mname != NULL);
	map = NULL;
	ret = sm_map_create(maps, mtype, 0, &map);
	SM_TEST_E(sm_is_success(ret));
	ret = sm_map_setopt(map, SMPO_HASH_NELEM, 23, SMPO_ELEM_MAX, 51,
			SMPO_END);
	SM_TEST_E(sm_is_success(ret));
	ret = sm_map_open(maps, mname, mtype, 0, NULL, SMAP_MODE_RDWR, &map,
			SMPO_END);
	SM_TEST_E(sm_is_success(ret));
	SM_CSTR_FREE(mtype);
	SM_CSTR_FREE(mname);

	*pmap = map;
	return SM_SUCCESS;

  error:
	*pmap = NULL;
	return ret;
}

#define SM_STR_CRT(var, value)	\
	do {	\
		sm_str_assign((var), NULL, (uchar *)(value), strlen((value)),	\
			strlen((value)));	\
	} while (0)

#define SM_POP_DB(map, Key, Data)	\
	do {	\
	SM_STR_CRT(key, (Key));	\
	SM_STR_CRT(data, (Data));	\
	ret = sm_map_add((map), &key, &data, SMMAP_AFL_NONE);	\
	SM_TEST(sm_is_success(ret));	\
	} while (0)

#define SM_POP_MAP1	\
	SM_POP_DB(map1, KEY1_1, DATA1_1);	\
	SM_POP_DB(map1, KEY1_2, DATA1_2);

#define SM_POP_MAP2	\
	SM_POP_DB(map2, KEY2_1, DATA2_1);	\
	SM_POP_DB(map2, KEY2_2, DATA2_2);

#define SM_POP_BDB2	\
	SM_POP_DB(map2r, KEY2_1, DATAR1);	\
	SM_POP_DB(map2r, KEY2_2, DATAR2);

#define SM_LOOKUP_TEST(map, Key1, Key2, Key3)	\
	do {	\
	sm_str_clr(rhs);	\
	SM_STR_CRT(key, Key1);	\
	ret = sm_map_lookup(map, SMMAP_FL_NONE, &key, rhs);	\
	SM_TEST(sm_is_success(ret));	\
	SM_STR_CRT(key, Key2);	\
	ret = sm_map_lookup(map, SMMAP_FL_NONE, &key, rhs);	\
	SM_TEST(sm_is_success(ret));	\
	SM_STR_CRT(key, Key3);	\
	ret = sm_map_lookup(map, SMMAP_FL_NONE, &key, rhs);	\
	SM_TEST(!sm_is_success(ret));	\
	} while (0)

#define SM_REWRITE_TEST(map, Key, Data)	\
	do {	\
	sm_str_clr(lhs);	\
	sm_str_clr(rhs);	\
	ret = sm_str_scat(lhs, (Key));	\
	SM_TEST(sm_is_success(ret));	\
	ret = sm_map_rewrite((map), SMMAP_FL_NONE, lhs, rhs);	\
	SM_TEST(sm_is_success(ret));	\
	SM_TEST(strncmp((Data), (char *) sm_str_data(rhs), sm_str_getlen(rhs)) == 0);\
	} while (0)

#define SM_CLOSE_MAPS	\
	ret = sm_map_close(map3, 0);	\
	SM_TEST(sm_is_success(ret));	\
	ret = sm_map_close(map2, 0);	\
	SM_TEST(sm_is_success(ret));	\
	ret = sm_map_close(map1, 0);	\
	SM_TEST(sm_is_success(ret))
