/*
 * Copyright (c) 2011 Sendmail, Inc. and its suppliers.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: dastate2flag.c,v 1.4 2011/05/19 17:49:40 ca Exp $")
#include "sm/common.h"
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/actdb-int.h"
#include "sm/da.h"

/*
**  DASTATE2FLAG -- Translate delivery error state into flag
**
**	Parameters:
**		errstate -- error state
**		pflag -- (pointer to) flag (output)
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
dastate2flag(uint errstate, uint32_t *pflag)
{
	sm_ret_T ret;

	SM_REQUIRE(pflag != NULL);
	ret = SM_SUCCESS;

/*
 *	not yet used/set:
 *		*pflag |= DE_TLS_E;
 *		*pflag |= DE_RSAD;
 *		*pflag |= DE_QUIT;
 */

	/*
	 *  should this distinguish error types?
	 *  See da.h
	 *  I: internal
	 *  W: during write
	 *  S: SMTP reply code
	 *  R: during read of reply
	 */

	switch (da_err_cmd(errstate)) {
	  case DA_TA_ERR_MAIL:
		*pflag |= DE_MAIL;
		break;
	  case DA_TA_ERR_RCPT:
		*pflag |= DE_RCPT;
		break;
	  case DA_TA_ERR_DATA:
		*pflag |= DE_DATA;
		break;
	  case DA_TA_ERR_BODY:
		break;
	  case DA_TA_ERR_DOT:
		*pflag |= DE_DOT;
		break;
	  case DA_TA_ERR_L_RCPT:
		*pflag |= DE_L_RCPT;
		break;
	  case DA_SE_ERR_OPEN:
		*pflag |= DE_CONN;
		break;
	  case DA_SE_ERR_GRET:
		*pflag |= DE_INIT;
		break;
	  case DA_SE_ERR_EHLO:
		*pflag |= DE_EHLO;
		break;
	  case DA_SE_ERR_HELO:
		*pflag |= DE_HELO;
		break;
	  case DA_SE_ERR_STLS:
		*pflag |= DE_TLS;
		break;
	  default:
		break;
	}

	return ret;
}
